# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Microscope.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'MicroscopeType' not in _M_omero.model.__dict__:
    _M_omero.model._t_MicroscopeType = IcePy.declareClass('::omero::model::MicroscopeType')
    _M_omero.model._t_MicroscopeTypePrx = IcePy.declareProxy('::omero::model::MicroscopeType')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'Microscope' not in _M_omero.model.__dict__:
    _M_omero.model.Microscope = Ice.createTempClass()
    class Microscope(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _manufacturer=None, _model=None, _lotNumber=None, _serialNumber=None, _type=None):
            if Ice.getType(self) == _M_omero.model.Microscope:
                raise RuntimeError('omero.model.Microscope is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._manufacturer = _manufacturer
            self._model = _model
            self._lotNumber = _lotNumber
            self._serialNumber = _serialNumber
            self._type = _type

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Microscope')

        def ice_id(self, current=None):
            return '::omero::model::Microscope'

        def ice_staticId():
            return '::omero::model::Microscope'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getManufacturer(self, current=None):
            pass

        def setManufacturer(self, theManufacturer, current=None):
            pass

        def getModel(self, current=None):
            pass

        def setModel(self, theModel, current=None):
            pass

        def getLotNumber(self, current=None):
            pass

        def setLotNumber(self, theLotNumber, current=None):
            pass

        def getSerialNumber(self, current=None):
            pass

        def setSerialNumber(self, theSerialNumber, current=None):
            pass

        def getType(self, current=None):
            pass

        def setType(self, theType, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Microscope)

        __repr__ = __str__

    _M_omero.model.MicroscopePrx = Ice.createTempClass()
    class MicroscopePrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Microscope._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Microscope._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Microscope._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Microscope._op_setVersion.end(self, _r)

        def getManufacturer(self, _ctx=None):
            return _M_omero.model.Microscope._op_getManufacturer.invoke(self, ((), _ctx))

        def begin_getManufacturer(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_getManufacturer.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getManufacturer(self, _r):
            return _M_omero.model.Microscope._op_getManufacturer.end(self, _r)

        def setManufacturer(self, theManufacturer, _ctx=None):
            return _M_omero.model.Microscope._op_setManufacturer.invoke(self, ((theManufacturer, ), _ctx))

        def begin_setManufacturer(self, theManufacturer, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_setManufacturer.begin(self, ((theManufacturer, ), _response, _ex, _sent, _ctx))

        def end_setManufacturer(self, _r):
            return _M_omero.model.Microscope._op_setManufacturer.end(self, _r)

        def getModel(self, _ctx=None):
            return _M_omero.model.Microscope._op_getModel.invoke(self, ((), _ctx))

        def begin_getModel(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_getModel.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getModel(self, _r):
            return _M_omero.model.Microscope._op_getModel.end(self, _r)

        def setModel(self, theModel, _ctx=None):
            return _M_omero.model.Microscope._op_setModel.invoke(self, ((theModel, ), _ctx))

        def begin_setModel(self, theModel, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_setModel.begin(self, ((theModel, ), _response, _ex, _sent, _ctx))

        def end_setModel(self, _r):
            return _M_omero.model.Microscope._op_setModel.end(self, _r)

        def getLotNumber(self, _ctx=None):
            return _M_omero.model.Microscope._op_getLotNumber.invoke(self, ((), _ctx))

        def begin_getLotNumber(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_getLotNumber.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLotNumber(self, _r):
            return _M_omero.model.Microscope._op_getLotNumber.end(self, _r)

        def setLotNumber(self, theLotNumber, _ctx=None):
            return _M_omero.model.Microscope._op_setLotNumber.invoke(self, ((theLotNumber, ), _ctx))

        def begin_setLotNumber(self, theLotNumber, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_setLotNumber.begin(self, ((theLotNumber, ), _response, _ex, _sent, _ctx))

        def end_setLotNumber(self, _r):
            return _M_omero.model.Microscope._op_setLotNumber.end(self, _r)

        def getSerialNumber(self, _ctx=None):
            return _M_omero.model.Microscope._op_getSerialNumber.invoke(self, ((), _ctx))

        def begin_getSerialNumber(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_getSerialNumber.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSerialNumber(self, _r):
            return _M_omero.model.Microscope._op_getSerialNumber.end(self, _r)

        def setSerialNumber(self, theSerialNumber, _ctx=None):
            return _M_omero.model.Microscope._op_setSerialNumber.invoke(self, ((theSerialNumber, ), _ctx))

        def begin_setSerialNumber(self, theSerialNumber, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_setSerialNumber.begin(self, ((theSerialNumber, ), _response, _ex, _sent, _ctx))

        def end_setSerialNumber(self, _r):
            return _M_omero.model.Microscope._op_setSerialNumber.end(self, _r)

        def getType(self, _ctx=None):
            return _M_omero.model.Microscope._op_getType.invoke(self, ((), _ctx))

        def begin_getType(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_getType.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getType(self, _r):
            return _M_omero.model.Microscope._op_getType.end(self, _r)

        def setType(self, theType, _ctx=None):
            return _M_omero.model.Microscope._op_setType.invoke(self, ((theType, ), _ctx))

        def begin_setType(self, theType, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Microscope._op_setType.begin(self, ((theType, ), _response, _ex, _sent, _ctx))

        def end_setType(self, _r):
            return _M_omero.model.Microscope._op_setType.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.MicroscopePrx.ice_checkedCast(proxy, '::omero::model::Microscope', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.MicroscopePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Microscope'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_MicroscopePrx = IcePy.defineProxy('::omero::model::Microscope', MicroscopePrx)

    _M_omero.model._t_Microscope = IcePy.declareClass('::omero::model::Microscope')

    _M_omero.model._t_Microscope = IcePy.defineClass('::omero::model::Microscope', Microscope, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_manufacturer', (), _M_omero._t_RString, False, 0),
        ('_model', (), _M_omero._t_RString, False, 0),
        ('_lotNumber', (), _M_omero._t_RString, False, 0),
        ('_serialNumber', (), _M_omero._t_RString, False, 0),
        ('_type', (), _M_omero.model._t_MicroscopeType, False, 0)
    ))
    Microscope._ice_type = _M_omero.model._t_Microscope

    Microscope._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Microscope._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Microscope._op_getManufacturer = IcePy.Operation('getManufacturer', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Microscope._op_setManufacturer = IcePy.Operation('setManufacturer', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Microscope._op_getModel = IcePy.Operation('getModel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Microscope._op_setModel = IcePy.Operation('setModel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Microscope._op_getLotNumber = IcePy.Operation('getLotNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Microscope._op_setLotNumber = IcePy.Operation('setLotNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Microscope._op_getSerialNumber = IcePy.Operation('getSerialNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Microscope._op_setSerialNumber = IcePy.Operation('setSerialNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Microscope._op_getType = IcePy.Operation('getType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_MicroscopeType, False, 0), ())
    Microscope._op_setType = IcePy.Operation('setType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_MicroscopeType, False, 0),), (), None, ())

    _M_omero.model.Microscope = Microscope
    del Microscope

    _M_omero.model.MicroscopePrx = MicroscopePrx
    del MicroscopePrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2009 Glencoe Software, Inc.  All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#

"""
OMERO Grid Processor
"""
from __future__ import division
from builtins import str
from builtins import range
from future.utils import native_str
from past.utils import old_div
import Ice
import time
import traceback

import omero  # Do we need both??
import omero.clients
import omero.callbacks
from omero_ext.path import path

# For ease of use
from omero import LockTimeout
from omero.rtypes import rstring
from omero.rtypes import unwrap
from omero.util.decorators import remoted, perf


sys = __import__("sys")  # Python sys
tables = __import__("tables")  # Pytables

VERSION = '2'
RETRIES = 20


def slen(rv):
    """
    Returns the length of the argument or None
    if the argument is None
    """
    if rv is None:
        return None
    return len(rv)


class TableI(omero.grid.Table, omero.util.SimpleServant):

    """
    Spreadsheet implementation based on pytables.
    """

    def __init__(self, ctx, file_obj, file_path, factory, storage_factory, read_only=False, uuid="unknown",
                 call_context=None, adapter=None):
        self.id = Ice.Identity()
        self.id.name = uuid
        self.uuid = uuid
        self.file_obj = file_obj
        self.factory = factory
        self.storage = storage_factory.getOrCreate(file_path, self, read_only)
        self.call_context = call_context
        self.adapter = adapter
        self.can_write = factory.getAdminService().canUpdate(
            file_obj, call_context)
        omero.util.SimpleServant.__init__(self, ctx)

        self.stamp = time.time()
        self._closed = False

        if (not self.file_obj.isLoaded() or
                self.file_obj.getDetails() is None or
                self.file_obj.details.group is None):
            self.file_obj = self.ctx.getSession().getQueryService().get(
                'omero.model.OriginalFileI', unwrap(file_obj.id),
                {"omero.group": "-1"})

    def assert_write(self):
        """
        Checks that the current user can write to the given object
        at the database level. If not, no FS level writes are permitted
        either.

        ticket:2910
        """
        if not self.can_write:
            raise omero.SecurityViolation(
                "Current user cannot write to file %s" % self.file_obj.id.val)

    def check(self):
        """
        Called periodically to check the resource is alive. Returns
        False if this resource can be cleaned up. (Resources API)
        """
        self.logger.debug("Checking %s" % self)
        if self._closed:
            return False

        idname = 'UNKNOWN'
        try:
            # Quietly loading the session will mean that the last access
            # time will not be incremented so that dangling files can be
            # cleaned up. Note: this is different that the strategy of a
            # script which *wants* to keep its session alive.
            idname = self.factory.ice_getIdentity().name
            clientSession = self.ctx.getSession().getSessionService() \
                .getSession(idname, {"quietly": "true"})
            if clientSession.getClosed():
                self.logger.debug("Client session closed: %s" % idname)
                return False
            return True
        except Exception:
            self.logger.debug("Client session not found: %s" % idname)
            return False

    def cleanup(self):
        """
        Decrements the counter on the held storage to allow it to
        be cleaned up. Returns the current file-size.
        """
        if self.storage:
            try:
                self.storage.decr(self)
                return self.storage.size()
            finally:
                self.storage = None

    def __str__(self):
        return "Table-%s" % self.uuid

    @remoted
    @perf
    def close(self, current=None):
        try:
            self.__close_file()
        finally:
            if self.adapter is not None:
                self.adapter.remove(self.id)
            self.adapter = None

    def __close_file(self):
        if self._closed:
            self.logger.warn(
                "File object %d already closed",
                unwrap(self.file_obj.id) if self.file_obj else None)
            return

        modified = self.storage.modified()

        try:
            size = self.cleanup()
            self.logger.info("Closed %s", self)
        except:
            self.logger.warn("Closed %s with errors", self)

        self._closed = True

        fid = unwrap(self.file_obj.id)

        if self.file_obj is not None and self.can_write and modified:
            gid = unwrap(self.file_obj.details.group.id)
            client_uuid = self.factory.ice_getIdentity().category[8:]
            ctx = {
                "omero.group": native_str(gid),
                omero.constants.CLIENTUUID: client_uuid}
            try:
                # Size to reset the server object to (must be checked after
                # the underlying HDF file has been closed)
                rfs = self.factory.createRawFileStore(ctx)
                try:
                    rfs.setFileId(fid, ctx)
                    if size:
                        rfs.truncate(size, ctx)     # May do nothing
                        rfs.write([], size, 0, ctx)  # Force an update
                    else:
                        rfs.write([], 0, 0, ctx)    # No-op
                    file_obj = rfs.save(ctx)
                finally:
                    rfs.close(ctx)
                self.logger.info(
                    "Updated file object %s to hash=%s (%s bytes)",
                    fid, unwrap(file_obj.hash), unwrap(file_obj.size))
            except:
                self.logger.warn("Failed to update file object %s",
                                 fid, exc_info=1)
        else:
            self.logger.info("File object %s not updated", fid)

    # TABLES READ API ============================

    @remoted
    @perf
    def getOriginalFile(self, current=None):
        msg = "unknown"
        if self.file_obj:
            if self.file_obj.id:
                msg = self.file_obj.id.val
        self.logger.info("%s.getOriginalFile() => id=%s", self, msg)
        return self.file_obj

    @remoted
    @perf
    def getHeaders(self, current=None):
        rv = self.storage.cols(None, current)
        self.logger.info("%s.getHeaders() => size=%s", self, slen(rv))
        return rv

    @remoted
    @perf
    def getNumberOfRows(self, current=None):
        rv = self.storage.rows()
        self.logger.info("%s.getNumberOfRows() => %s", self, rv)
        return int(rv)

    @remoted
    @perf
    def getWhereList(self, condition, variables,
                     start, stop, step, current=None):
        variables = unwrap(variables)
        if stop == 0:
            stop = None
        if step == 0:
            step = None
        rv = self.storage.getWhereList(
            self.stamp, condition, variables, None, start, stop, step)
        self.logger.info("%s.getWhereList(%s, %s, %s, %s, %s) => size=%s",
                         self, condition, variables,
                         start, stop, step, slen(rv))
        return rv

    @remoted
    @perf
    def readCoordinates(self, rowNumbers, current=None):
        self.logger.info("%s.readCoordinates(size=%s)", self, slen(rowNumbers))
        try:
            return self.storage.readCoordinates(self.stamp, rowNumbers,
                                                current)
        except tables.HDF5ExtError as err:
            aue = omero.ApiUsageException()
            aue.message = "Error reading coordinates. Most likely out of range"
            aue.serverStackTrace = "".join(traceback.format_exc())
            aue.serverExceptionClass = str(err.__class__.__name__)
            raise aue

    @remoted
    @perf
    def read(self, colNumbers, start, stop, current=None):
        self.logger.info("%s.read(%s, %s, %s)", self, colNumbers, start, stop)
        if start == 0 and stop == 0:
            stop = None
        try:
            return self.storage.read(self.stamp, colNumbers,
                                     start, stop, current)
        except tables.HDF5ExtError as err:
            aue = omero.ApiUsageException()
            aue.message = "Error reading coordinates. Most likely out of range"
            aue.serverStackTrace = "".join(traceback.format_exc())
            aue.serverExceptionClass = str(err.__class__.__name__)
            raise aue

    @remoted
    @perf
    def slice(self, colNumbers, rowNumbers, current=None):
        self.logger.info(
            "%s.slice(size=%s, size=%s)", self,
            slen(colNumbers), slen(rowNumbers))
        return self.storage.slice(self.stamp, colNumbers, rowNumbers, current)

    # TABLES WRITE API ===========================

    @remoted
    @perf
    def initialize(self, cols, current=None):
        self.assert_write()
        self.storage.initialize(cols)
        if cols:
            self.logger.info("Initialized %s with %s col(s)", self, slen(cols))

    @remoted
    @perf
    def addColumn(self, col, current=None):
        self.assert_write()
        raise omero.ApiUsageException(None, None, "NYI")

    @remoted
    @perf
    def addData(self, cols, current=None):
        self.assert_write()
        self.storage.append(cols)
        if cols and cols[0] and cols[0].getsize():
            self.logger.info(
                "Added %s row(s) of data to %s", cols[0].getsize(), self)

    @remoted
    @perf
    def update(self, data, current=None):
        self.assert_write()
        if data:
            self.storage.update(self.stamp, data)
            self.logger.info(
                "Updated %s row(s) of data to %s", slen(data.rowNumbers), self)

    @remoted
    @perf
    def delete(self, current=None):
        self.assert_write()
        self.close()
        dc = omero.cmd.Delete2(
            targetObjects={"OriginalFile": [self.file_obj.id.val]}
        )
        handle = self.factory.submit(dc)
        # Copied from clients.py since none is available
        try:
            callback = omero.callbacks.CmdCallbackI(
                current.adapter, handle, "Fake")
        except:
            # Since the callback won't escape this method,
            # close the handle if requested.
            handle.close()
            raise

        try:
            try:
                callback.loop(20, 500)
            except LockTimeout:
                raise omero.InternalException(None, None, "delete timed-out")

            rsp = callback.getResponse()
            if isinstance(rsp, omero.cmd.ERR):
                raise omero.InternalException(None, None, str(rsp))
        finally:
            callback.close(True)

    # TABLES METADATA API ===========================

    @remoted
    @perf
    def getMetadata(self, key, current=None):
        rv = self.storage.get_meta_map()
        rv = rv.get(key)
        self.logger.info("%s.getMetadata() => %s", self, unwrap(rv))
        return rv

    @remoted
    @perf
    def getAllMetadata(self, current=None):
        rv = self.storage.get_meta_map()
        self.logger.info("%s.getMetadata() => size=%s", self, slen(rv))
        return rv

    @remoted
    @perf
    def setMetadata(self, key, value, current=None):
        self.assert_write()
        self.storage.add_meta_map({key: value})
        self.logger.info("%s.setMetadata() => %s=%s", self, key, unwrap(value))

    @remoted
    @perf
    def setAllMetadata(self, value, current=None):
        self.assert_write()
        self.storage.add_meta_map(value, replace=True)
        self.logger.info("%s.setMetadata() => number=%s", self, slen(value))

    # Column methods missing


class TablesI(omero.grid.Tables, omero.util.Servant):

    """
    Implementation of the omero.grid.Tables API. Provides
    spreadsheet like functionality across the OMERO.grid.
    This servant serves as a session-less, user-less
    resource for obtaining omero.grid.Table proxies.

    The first major step in initialization is getting
    a session. This will block until the Blitz server
    is reachable.
    """

    def __init__(
            self, ctx,
            table_cast=omero.grid.TablePrx.uncheckedCast,
            internal_repo_cast=omero.grid.InternalRepositoryPrx.checkedCast,
            storage_factory=None,
            retries=None):

        omero.util.Servant.__init__(self, ctx, needs_session=True)

        # Storing these methods, mainly to allow overriding via
        # test methods. Static methods are evil.
        self._table_cast = table_cast
        self._internal_repo_cast = internal_repo_cast

        self.__stores = []

        if storage_factory is None:
            from omero.hdfstorageV2 import HDFLIST
            self._storage_factory = HDFLIST
        else:
            self._storage_factory = storage_factory
        self.logger.info("Using storage factory: %s.%s",
                         str(self._storage_factory.__module__),
                         self._storage_factory.__class__.__name__)

        self.repo_cfg = None
        self.repo_mgr = None
        self.repo_obj = None
        self.repo_svc = None
        self.repo_uuid = None

        try:
            config_service = ctx.getSession().getConfigService()
            prefix = "omero.cluster.read_only.runtime"
            self.read_only = "true" in [config_service.getConfigValue(
                "{}.{}".format(prefix, suffix)) for suffix in ["db", "repo"]]
        except:
            self.read_only = False

        if self.read_only:
            self.logger.info("Starting in read-only mode.")

        if retries is None:
            retries = RETRIES

        wait = float(self.communicator.getProperties().getPropertyWithDefault(
            "omero.repo.wait", "1"))
        per_loop = old_div(wait, retries)

        exc = None
        for x in range(retries):
            try:
                self._get_dir()
                self._get_uuid()
                self._get_repo()
            except Exception as e:
                self.logger.warn("Failed to find repo_svc: %s" % e)
                exc = e

            if self.repo_svc:
                break
            else:
                msg = "waiting %ss (%s of %s)" % (per_loop, x+1, retries)
                self.logger.debug(msg)
                self.stop_event.wait(per_loop)

        if exc:
            raise exc

    def _get_dir(self):
        """
        Second step in initialization is to find the .omero/repository
        directory. If this is not created, then a required server has
        not started, and so this instance will not start.
        """
        self.repo_dir = self.communicator.getProperties().getProperty(
            "omero.repo.dir")

        if not self.repo_dir:
            # Implies this is the legacy directory. Obtain from server
            self.repo_dir = self.ctx.getSession(
                ).getConfigService().getConfigValue("omero.data.dir")

        self.repo_cfg = path(self.repo_dir) / ".omero" / "repository"
        if not self.repo_cfg.exists():
            msg = "No repository found: %s" % self.repo_cfg
            raise omero.ResourceError(None, None, msg)

    def _get_uuid(self):
        """
        Third step in initialization is to find the database uuid
        for this grid instance. Multiple OMERO.grids could be watching
        the same directory.
        """
        cfg = self.ctx.getSession().getConfigService()
        self.db_uuid = cfg.getDatabaseUuid()
        self.instance = old_div(self.repo_cfg, self.db_uuid)

    def _get_repo(self):
        """
        Fourth step in initialization is to find the repository object
        for the UUID found in .omero/repository/<db_uuid>, and then
        create a proxy for the InternalRepository attached to that.
        """

        uuidfile = old_div(self.instance, "repo_uuid")
        if not uuidfile.exists():
            msg = "%s doesn't exist" % uuidfile
            raise IOError(msg)

        # Get and parse the uuid from the RandomAccessFile format from
        # FileMaker
        self.repo_uuid = uuidfile.lines()[0].strip()
        if len(self.repo_uuid) != 38:
            raise omero.ResourceError(
                "Poorly formed UUID: %s" % self.repo_uuid)
        self.repo_uuid = self.repo_uuid[2:]

        # Using the repo_uuid, find our OriginalFile object
        self.repo_obj = self.ctx.getSession().getQueryService().findByQuery(
            "select f from OriginalFile f where hash = :uuid",
            omero.sys.ParametersI().add("uuid", rstring(self.repo_uuid)))
        self.repo_mgr = self.communicator.stringToProxy(
            "InternalRepository-%s" % self.repo_uuid)
        self.repo_mgr = self._internal_repo_cast(self.repo_mgr)
        self.repo_svc = self.repo_mgr.getProxy()

    @remoted
    def getRepository(self, current=None):
        """
        Returns the Repository object for this Tables server.
        """
        return self.repo_svc

    @remoted
    @perf
    def getTable(self, file_obj, factory, current=None):
        """
        Create and/or register a table servant.
        """

        # Will throw an exception if not allowed.
        file_id = None
        if file_obj is not None and file_obj.id is not None:
            file_id = file_obj.id.val
        self.logger.info("getTable: %s %s", file_id, current.ctx)

        file_path = self.repo_mgr.getFilePath(file_obj)
        p = path(file_path).dirname()
        if not p.exists():
            p.makedirs()

        table = TableI(self.ctx, file_obj,file_path,
                       factory,
                       self._storage_factory,
                       read_only=self.read_only,
                       uuid=Ice.generateUUID(),
                       call_context=current.ctx,
                       adapter=current.adapter)
        self.resources.add(table)
        prx = current.adapter.add(table, table.id)
        return self._table_cast(prx)



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Permissions.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_RTypes_ice
import omero_ModelF_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Permissions' not in _M_omero.model.__dict__:
    _M_omero.model.Permissions = Ice.createTempClass()
    class Permissions(Ice.Object):
        """
        Row-level permissions definition available on
        every OMERO.blitz type. Represents a similar
        logic to the Unix filesystem.
        Members:
        restrictions -- Restrictions placed on the current object for the current
        user. Indexes into this array are based on constants
        in the {@code omero.constants.permissions} module. If a
        restriction index is not present, then it is safe to
        assume that there is no such restriction.
        If null, this should be assumed to have no restrictions.
        extendedRestrictions -- Further restrictions which are specified by services
        at runtime. Individual service methods will specify
        which strings MAY NOT be present in this field for
        execution to be successful. For example, if an
        omero.model.Image contains a ""DOWNLOAD"" restriction,
        then an attempt to call {@code omero.api.RawFileStore.read}
        will fail with an omero.SecurityViolation.
        perm1 -- Internal representation. May change!
        To make working with this object more straight-forward
        accessors are provided for the perm1 instance though it
        is protected, though NO GUARANTEES are made on the
        representation.
        """
        def __init__(self, _restrictions=None, _extendedRestrictions=None, _perm1=0):
            if Ice.getType(self) == _M_omero.model.Permissions:
                raise RuntimeError('omero.model.Permissions is an abstract class')
            self._restrictions = _restrictions
            self._extendedRestrictions = _extendedRestrictions
            self._perm1 = _perm1

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::Permissions')

        def ice_id(self, current=None):
            return '::omero::model::Permissions'

        def ice_staticId():
            return '::omero::model::Permissions'
        ice_staticId = staticmethod(ice_staticId)

        def getPerm1(self, current=None):
            """
            Do not use!
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def setPerm1(self, value, current=None):
            """
            Do not use!
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def isDisallow(self, restriction, current=None):
            """
            The basis for the other canX() methods. If the restriction
            at the given offset in the restriction array is true, then
            this method returns true (otherwise false) and the canX()
            methods return the opposite, i.e.
            isDisallow(ANNOTATERESTRICTION) == ! canAnnotate()
            Arguments:
            restriction -- 
            current -- The Current object for the invocation.
            """
            pass

        def isRestricted(self, restriction, current=None):
            """
            Returns true if the given argument is present in the
            extendedRestrictions set. This implies that some
            service-specific behavior is disallowed.
            Arguments:
            restriction -- 
            current -- The Current object for the invocation.
            """
            pass

        def canAnnotate(self, current=None):
            """
            Whether the current user has permissions
            for annotating this object.
            The fact that the user has this object in hand
            already identifies that it's readable.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def canEdit(self, current=None):
            """
            Whether the current user has the ""edit"" permissions
            for this object. This includes changing the values
            of the object.
            The fact that the user has this object in hand
            already identifies that it's readable.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def canLink(self, current=None):
            """
            Whether the current user has the ""link"" permissions
            for this object. This includes adding it to data graphs.
            The fact that the user has this object in hand
            already identifies that it's readable.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def canDelete(self, current=None):
            """
            Whether the current user has the ""delete"" permissions
            for this object.
            The fact that the user has this object in hand
            already identifies that it's readable.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def canChgrp(self, current=None):
            """
            Whether the current user has the ""chgrp"" permissions
            for this object. This allows them to move it to a different group.
            The fact that the user has this object in hand
            already identifies that it's readable.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def canChown(self, current=None):
            """
            Whether the current user has the ""chown"" permissions
            for this object. This allows them to give it to a different user.
            The fact that the user has this object in hand
            already identifies that it's readable.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def isUserRead(self, current=None):
            pass

        def isUserAnnotate(self, current=None):
            pass

        def isUserWrite(self, current=None):
            pass

        def isGroupRead(self, current=None):
            pass

        def isGroupAnnotate(self, current=None):
            pass

        def isGroupWrite(self, current=None):
            pass

        def isWorldRead(self, current=None):
            pass

        def isWorldAnnotate(self, current=None):
            pass

        def isWorldWrite(self, current=None):
            pass

        def setUserRead(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setUserAnnotate(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setUserWrite(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setGroupRead(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setGroupAnnotate(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setGroupWrite(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setWorldRead(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setWorldAnnotate(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def setWorldWrite(self, value, current=None):
            """
            Throws omero.ClientError if mutation not allowed.
            Arguments:
            value -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Permissions)

        __repr__ = __str__

    _M_omero.model.PermissionsPrx = Ice.createTempClass()
    class PermissionsPrx(Ice.ObjectPrx):

        """
        Do not use!
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getPerm1(self, _ctx=None):
            return _M_omero.model.Permissions._op_getPerm1.invoke(self, ((), _ctx))

        """
        Do not use!
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPerm1(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_getPerm1.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Do not use!
        Arguments:
        """
        def end_getPerm1(self, _r):
            return _M_omero.model.Permissions._op_getPerm1.end(self, _r)

        """
        Do not use!
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setPerm1(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setPerm1.invoke(self, ((value, ), _ctx))

        """
        Do not use!
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setPerm1(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setPerm1.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Do not use!
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setPerm1(self, _r):
            return _M_omero.model.Permissions._op_setPerm1.end(self, _r)

        """
        The basis for the other canX() methods. If the restriction
        at the given offset in the restriction array is true, then
        this method returns true (otherwise false) and the canX()
        methods return the opposite, i.e.
        isDisallow(ANNOTATERESTRICTION) == ! canAnnotate()
        Arguments:
        restriction -- 
        _ctx -- The request context for the invocation.
        """
        def isDisallow(self, restriction, _ctx=None):
            return _M_omero.model.Permissions._op_isDisallow.invoke(self, ((restriction, ), _ctx))

        """
        The basis for the other canX() methods. If the restriction
        at the given offset in the restriction array is true, then
        this method returns true (otherwise false) and the canX()
        methods return the opposite, i.e.
        isDisallow(ANNOTATERESTRICTION) == ! canAnnotate()
        Arguments:
        restriction -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isDisallow(self, restriction, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isDisallow.begin(self, ((restriction, ), _response, _ex, _sent, _ctx))

        """
        The basis for the other canX() methods. If the restriction
        at the given offset in the restriction array is true, then
        this method returns true (otherwise false) and the canX()
        methods return the opposite, i.e.
        isDisallow(ANNOTATERESTRICTION) == ! canAnnotate()
        Arguments:
        restriction -- 
        """
        def end_isDisallow(self, _r):
            return _M_omero.model.Permissions._op_isDisallow.end(self, _r)

        """
        Returns true if the given argument is present in the
        extendedRestrictions set. This implies that some
        service-specific behavior is disallowed.
        Arguments:
        restriction -- 
        _ctx -- The request context for the invocation.
        """
        def isRestricted(self, restriction, _ctx=None):
            return _M_omero.model.Permissions._op_isRestricted.invoke(self, ((restriction, ), _ctx))

        """
        Returns true if the given argument is present in the
        extendedRestrictions set. This implies that some
        service-specific behavior is disallowed.
        Arguments:
        restriction -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isRestricted(self, restriction, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isRestricted.begin(self, ((restriction, ), _response, _ex, _sent, _ctx))

        """
        Returns true if the given argument is present in the
        extendedRestrictions set. This implies that some
        service-specific behavior is disallowed.
        Arguments:
        restriction -- 
        """
        def end_isRestricted(self, _r):
            return _M_omero.model.Permissions._op_isRestricted.end(self, _r)

        """
        Whether the current user has permissions
        for annotating this object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def canAnnotate(self, _ctx=None):
            return _M_omero.model.Permissions._op_canAnnotate.invoke(self, ((), _ctx))

        """
        Whether the current user has permissions
        for annotating this object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_canAnnotate(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_canAnnotate.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether the current user has permissions
        for annotating this object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        """
        def end_canAnnotate(self, _r):
            return _M_omero.model.Permissions._op_canAnnotate.end(self, _r)

        """
        Whether the current user has the ""edit"" permissions
        for this object. This includes changing the values
        of the object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def canEdit(self, _ctx=None):
            return _M_omero.model.Permissions._op_canEdit.invoke(self, ((), _ctx))

        """
        Whether the current user has the ""edit"" permissions
        for this object. This includes changing the values
        of the object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_canEdit(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_canEdit.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether the current user has the ""edit"" permissions
        for this object. This includes changing the values
        of the object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        """
        def end_canEdit(self, _r):
            return _M_omero.model.Permissions._op_canEdit.end(self, _r)

        """
        Whether the current user has the ""link"" permissions
        for this object. This includes adding it to data graphs.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def canLink(self, _ctx=None):
            return _M_omero.model.Permissions._op_canLink.invoke(self, ((), _ctx))

        """
        Whether the current user has the ""link"" permissions
        for this object. This includes adding it to data graphs.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_canLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_canLink.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether the current user has the ""link"" permissions
        for this object. This includes adding it to data graphs.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        """
        def end_canLink(self, _r):
            return _M_omero.model.Permissions._op_canLink.end(self, _r)

        """
        Whether the current user has the ""delete"" permissions
        for this object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def canDelete(self, _ctx=None):
            return _M_omero.model.Permissions._op_canDelete.invoke(self, ((), _ctx))

        """
        Whether the current user has the ""delete"" permissions
        for this object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_canDelete(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_canDelete.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether the current user has the ""delete"" permissions
        for this object.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        """
        def end_canDelete(self, _r):
            return _M_omero.model.Permissions._op_canDelete.end(self, _r)

        """
        Whether the current user has the ""chgrp"" permissions
        for this object. This allows them to move it to a different group.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def canChgrp(self, _ctx=None):
            return _M_omero.model.Permissions._op_canChgrp.invoke(self, ((), _ctx))

        """
        Whether the current user has the ""chgrp"" permissions
        for this object. This allows them to move it to a different group.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_canChgrp(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_canChgrp.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether the current user has the ""chgrp"" permissions
        for this object. This allows them to move it to a different group.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        """
        def end_canChgrp(self, _r):
            return _M_omero.model.Permissions._op_canChgrp.end(self, _r)

        """
        Whether the current user has the ""chown"" permissions
        for this object. This allows them to give it to a different user.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def canChown(self, _ctx=None):
            return _M_omero.model.Permissions._op_canChown.invoke(self, ((), _ctx))

        """
        Whether the current user has the ""chown"" permissions
        for this object. This allows them to give it to a different user.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_canChown(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_canChown.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether the current user has the ""chown"" permissions
        for this object. This allows them to give it to a different user.
        The fact that the user has this object in hand
        already identifies that it's readable.
        Arguments:
        """
        def end_canChown(self, _r):
            return _M_omero.model.Permissions._op_canChown.end(self, _r)

        def isUserRead(self, _ctx=None):
            return _M_omero.model.Permissions._op_isUserRead.invoke(self, ((), _ctx))

        def begin_isUserRead(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isUserRead.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isUserRead(self, _r):
            return _M_omero.model.Permissions._op_isUserRead.end(self, _r)

        def isUserAnnotate(self, _ctx=None):
            return _M_omero.model.Permissions._op_isUserAnnotate.invoke(self, ((), _ctx))

        def begin_isUserAnnotate(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isUserAnnotate.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isUserAnnotate(self, _r):
            return _M_omero.model.Permissions._op_isUserAnnotate.end(self, _r)

        def isUserWrite(self, _ctx=None):
            return _M_omero.model.Permissions._op_isUserWrite.invoke(self, ((), _ctx))

        def begin_isUserWrite(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isUserWrite.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isUserWrite(self, _r):
            return _M_omero.model.Permissions._op_isUserWrite.end(self, _r)

        def isGroupRead(self, _ctx=None):
            return _M_omero.model.Permissions._op_isGroupRead.invoke(self, ((), _ctx))

        def begin_isGroupRead(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isGroupRead.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isGroupRead(self, _r):
            return _M_omero.model.Permissions._op_isGroupRead.end(self, _r)

        def isGroupAnnotate(self, _ctx=None):
            return _M_omero.model.Permissions._op_isGroupAnnotate.invoke(self, ((), _ctx))

        def begin_isGroupAnnotate(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isGroupAnnotate.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isGroupAnnotate(self, _r):
            return _M_omero.model.Permissions._op_isGroupAnnotate.end(self, _r)

        def isGroupWrite(self, _ctx=None):
            return _M_omero.model.Permissions._op_isGroupWrite.invoke(self, ((), _ctx))

        def begin_isGroupWrite(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isGroupWrite.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isGroupWrite(self, _r):
            return _M_omero.model.Permissions._op_isGroupWrite.end(self, _r)

        def isWorldRead(self, _ctx=None):
            return _M_omero.model.Permissions._op_isWorldRead.invoke(self, ((), _ctx))

        def begin_isWorldRead(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isWorldRead.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isWorldRead(self, _r):
            return _M_omero.model.Permissions._op_isWorldRead.end(self, _r)

        def isWorldAnnotate(self, _ctx=None):
            return _M_omero.model.Permissions._op_isWorldAnnotate.invoke(self, ((), _ctx))

        def begin_isWorldAnnotate(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isWorldAnnotate.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isWorldAnnotate(self, _r):
            return _M_omero.model.Permissions._op_isWorldAnnotate.end(self, _r)

        def isWorldWrite(self, _ctx=None):
            return _M_omero.model.Permissions._op_isWorldWrite.invoke(self, ((), _ctx))

        def begin_isWorldWrite(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_isWorldWrite.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_isWorldWrite(self, _r):
            return _M_omero.model.Permissions._op_isWorldWrite.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setUserRead(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setUserRead.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setUserRead(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setUserRead.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setUserRead(self, _r):
            return _M_omero.model.Permissions._op_setUserRead.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setUserAnnotate(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setUserAnnotate.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setUserAnnotate(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setUserAnnotate.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setUserAnnotate(self, _r):
            return _M_omero.model.Permissions._op_setUserAnnotate.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setUserWrite(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setUserWrite.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setUserWrite(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setUserWrite.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setUserWrite(self, _r):
            return _M_omero.model.Permissions._op_setUserWrite.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setGroupRead(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setGroupRead.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setGroupRead(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setGroupRead.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setGroupRead(self, _r):
            return _M_omero.model.Permissions._op_setGroupRead.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setGroupAnnotate(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setGroupAnnotate.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setGroupAnnotate(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setGroupAnnotate.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setGroupAnnotate(self, _r):
            return _M_omero.model.Permissions._op_setGroupAnnotate.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setGroupWrite(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setGroupWrite.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setGroupWrite(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setGroupWrite.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setGroupWrite(self, _r):
            return _M_omero.model.Permissions._op_setGroupWrite.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setWorldRead(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setWorldRead.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setWorldRead(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setWorldRead.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setWorldRead(self, _r):
            return _M_omero.model.Permissions._op_setWorldRead.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setWorldAnnotate(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setWorldAnnotate.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setWorldAnnotate(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setWorldAnnotate.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setWorldAnnotate(self, _r):
            return _M_omero.model.Permissions._op_setWorldAnnotate.end(self, _r)

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _ctx -- The request context for the invocation.
        """
        def setWorldWrite(self, value, _ctx=None):
            return _M_omero.model.Permissions._op_setWorldWrite.invoke(self, ((value, ), _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setWorldWrite(self, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Permissions._op_setWorldWrite.begin(self, ((value, ), _response, _ex, _sent, _ctx))

        """
        Throws omero.ClientError if mutation not allowed.
        Arguments:
        value -- 
        """
        def end_setWorldWrite(self, _r):
            return _M_omero.model.Permissions._op_setWorldWrite.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.PermissionsPrx.ice_checkedCast(proxy, '::omero::model::Permissions', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.PermissionsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Permissions'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_PermissionsPrx = IcePy.defineProxy('::omero::model::Permissions', PermissionsPrx)

    _M_omero.model._t_Permissions = IcePy.defineClass('::omero::model::Permissions', Permissions, -1, (), True, False, None, (), (
        ('_restrictions', (), _M_omero.api._t_BoolArray, False, 0),
        ('_extendedRestrictions', (), _M_omero.api._t_StringSet, False, 0),
        ('_perm1', (), IcePy._t_long, False, 0)
    ))
    Permissions._ice_type = _M_omero.model._t_Permissions

    Permissions._op_getPerm1 = IcePy.Operation('getPerm1', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_long, False, 0), ())
    Permissions._op_setPerm1 = IcePy.Operation('setPerm1', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_long, False, 0),), (), None, ())
    Permissions._op_isDisallow = IcePy.Operation('isDisallow', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0),), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isRestricted = IcePy.Operation('isRestricted', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_canAnnotate = IcePy.Operation('canAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_canEdit = IcePy.Operation('canEdit', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_canLink = IcePy.Operation('canLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_canDelete = IcePy.Operation('canDelete', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_canChgrp = IcePy.Operation('canChgrp', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_canChown = IcePy.Operation('canChown', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isUserRead = IcePy.Operation('isUserRead', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isUserAnnotate = IcePy.Operation('isUserAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isUserWrite = IcePy.Operation('isUserWrite', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isGroupRead = IcePy.Operation('isGroupRead', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isGroupAnnotate = IcePy.Operation('isGroupAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isGroupWrite = IcePy.Operation('isGroupWrite', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isWorldRead = IcePy.Operation('isWorldRead', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isWorldAnnotate = IcePy.Operation('isWorldAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_isWorldWrite = IcePy.Operation('isWorldWrite', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), ())
    Permissions._op_setUserRead = IcePy.Operation('setUserRead', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setUserAnnotate = IcePy.Operation('setUserAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setUserWrite = IcePy.Operation('setUserWrite', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setGroupRead = IcePy.Operation('setGroupRead', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setGroupAnnotate = IcePy.Operation('setGroupAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setGroupWrite = IcePy.Operation('setGroupWrite', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setWorldRead = IcePy.Operation('setWorldRead', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setWorldAnnotate = IcePy.Operation('setWorldAnnotate', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())
    Permissions._op_setWorldWrite = IcePy.Operation('setWorldWrite', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_bool, False, 0),), (), None, ())

    _M_omero.model.Permissions = Permissions
    del Permissions

    _M_omero.model.PermissionsPrx = PermissionsPrx
    del PermissionsPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Share.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice
import omero_model_Session_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'ExperimenterGroup' not in _M_omero.model.__dict__:
    _M_omero.model._t_ExperimenterGroup = IcePy.declareClass('::omero::model::ExperimenterGroup')
    _M_omero.model._t_ExperimenterGroupPrx = IcePy.declareProxy('::omero::model::ExperimenterGroup')

if 'Node' not in _M_omero.model.__dict__:
    _M_omero.model._t_Node = IcePy.declareClass('::omero::model::Node')
    _M_omero.model._t_NodePrx = IcePy.declareProxy('::omero::model::Node')

if 'Experimenter' not in _M_omero.model.__dict__:
    _M_omero.model._t_Experimenter = IcePy.declareClass('::omero::model::Experimenter')
    _M_omero.model._t_ExperimenterPrx = IcePy.declareProxy('::omero::model::Experimenter')

if 'Event' not in _M_omero.model.__dict__:
    _M_omero.model._t_Event = IcePy.declareClass('::omero::model::Event')
    _M_omero.model._t_EventPrx = IcePy.declareProxy('::omero::model::Event')

if 'SessionAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_SessionAnnotationLink = IcePy.declareClass('::omero::model::SessionAnnotationLink')
    _M_omero.model._t_SessionAnnotationLinkPrx = IcePy.declareProxy('::omero::model::SessionAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'Share' not in _M_omero.model.__dict__:
    _M_omero.model.Share = Ice.createTempClass()
    class Share(_M_omero.model.Session):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _node=None, _uuid=None, _owner=None, _sudoer=None, _timeToIdle=None, _timeToLive=None, _started=None, _closed=None, _message=None, _defaultEventType=None, _userAgent=None, _userIP=None, _eventsSeq=None, _eventsLoaded=False, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _group=None, _itemCount=None, _active=None, _data=None):
            if Ice.getType(self) == _M_omero.model.Share:
                raise RuntimeError('omero.model.Share is an abstract class')
            _M_omero.model.Session.__init__(self, _id, _details, _loaded, _version, _node, _uuid, _owner, _sudoer, _timeToIdle, _timeToLive, _started, _closed, _message, _defaultEventType, _userAgent, _userIP, _eventsSeq, _eventsLoaded, _annotationLinksSeq, _annotationLinksLoaded, _annotationLinksCountPerOwner)
            self._group = _group
            self._itemCount = _itemCount
            self._active = _active
            self._data = _data

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Session', '::omero::model::Share')

        def ice_id(self, current=None):
            return '::omero::model::Share'

        def ice_staticId():
            return '::omero::model::Share'
        ice_staticId = staticmethod(ice_staticId)

        def getGroup(self, current=None):
            pass

        def setGroup(self, theGroup, current=None):
            pass

        def getItemCount(self, current=None):
            pass

        def setItemCount(self, theItemCount, current=None):
            pass

        def getActive(self, current=None):
            pass

        def setActive(self, theActive, current=None):
            pass

        def getData(self, current=None):
            pass

        def setData(self, theData, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Share)

        __repr__ = __str__

    _M_omero.model.SharePrx = Ice.createTempClass()
    class SharePrx(_M_omero.model.SessionPrx):

        def getGroup(self, _ctx=None):
            return _M_omero.model.Share._op_getGroup.invoke(self, ((), _ctx))

        def begin_getGroup(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_getGroup.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getGroup(self, _r):
            return _M_omero.model.Share._op_getGroup.end(self, _r)

        def setGroup(self, theGroup, _ctx=None):
            return _M_omero.model.Share._op_setGroup.invoke(self, ((theGroup, ), _ctx))

        def begin_setGroup(self, theGroup, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_setGroup.begin(self, ((theGroup, ), _response, _ex, _sent, _ctx))

        def end_setGroup(self, _r):
            return _M_omero.model.Share._op_setGroup.end(self, _r)

        def getItemCount(self, _ctx=None):
            return _M_omero.model.Share._op_getItemCount.invoke(self, ((), _ctx))

        def begin_getItemCount(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_getItemCount.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getItemCount(self, _r):
            return _M_omero.model.Share._op_getItemCount.end(self, _r)

        def setItemCount(self, theItemCount, _ctx=None):
            return _M_omero.model.Share._op_setItemCount.invoke(self, ((theItemCount, ), _ctx))

        def begin_setItemCount(self, theItemCount, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_setItemCount.begin(self, ((theItemCount, ), _response, _ex, _sent, _ctx))

        def end_setItemCount(self, _r):
            return _M_omero.model.Share._op_setItemCount.end(self, _r)

        def getActive(self, _ctx=None):
            return _M_omero.model.Share._op_getActive.invoke(self, ((), _ctx))

        def begin_getActive(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_getActive.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getActive(self, _r):
            return _M_omero.model.Share._op_getActive.end(self, _r)

        def setActive(self, theActive, _ctx=None):
            return _M_omero.model.Share._op_setActive.invoke(self, ((theActive, ), _ctx))

        def begin_setActive(self, theActive, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_setActive.begin(self, ((theActive, ), _response, _ex, _sent, _ctx))

        def end_setActive(self, _r):
            return _M_omero.model.Share._op_setActive.end(self, _r)

        def getData(self, _ctx=None):
            return _M_omero.model.Share._op_getData.invoke(self, ((), _ctx))

        def begin_getData(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_getData.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getData(self, _r):
            return _M_omero.model.Share._op_getData.end(self, _r)

        def setData(self, theData, _ctx=None):
            return _M_omero.model.Share._op_setData.invoke(self, ((theData, ), _ctx))

        def begin_setData(self, theData, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Share._op_setData.begin(self, ((theData, ), _response, _ex, _sent, _ctx))

        def end_setData(self, _r):
            return _M_omero.model.Share._op_setData.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.SharePrx.ice_checkedCast(proxy, '::omero::model::Share', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.SharePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Share'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_SharePrx = IcePy.defineProxy('::omero::model::Share', SharePrx)

    _M_omero.model._t_Share = IcePy.declareClass('::omero::model::Share')

    _M_omero.model._t_Share = IcePy.defineClass('::omero::model::Share', Share, -1, (), True, False, _M_omero.model._t_Session, (), (
        ('_group', (), _M_omero.model._t_ExperimenterGroup, False, 0),
        ('_itemCount', (), _M_omero._t_RLong, False, 0),
        ('_active', (), _M_omero._t_RBool, False, 0),
        ('_data', (), _M_Ice._t_ByteSeq, False, 0)
    ))
    Share._ice_type = _M_omero.model._t_Share

    Share._op_getGroup = IcePy.Operation('getGroup', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ExperimenterGroup, False, 0), ())
    Share._op_setGroup = IcePy.Operation('setGroup', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroup, False, 0),), (), None, ())
    Share._op_getItemCount = IcePy.Operation('getItemCount', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RLong, False, 0), ())
    Share._op_setItemCount = IcePy.Operation('setItemCount', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RLong, False, 0),), (), None, ())
    Share._op_getActive = IcePy.Operation('getActive', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RBool, False, 0), ())
    Share._op_setActive = IcePy.Operation('setActive', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RBool, False, 0),), (), None, ())
    Share._op_getData = IcePy.Operation('getData', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_Ice._t_ByteSeq, False, 0), ())
    Share._op_setData = IcePy.Operation('setData', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_ByteSeq, False, 0),), (), None, ())

    _M_omero.model.Share = Share
    del Share

    _M_omero.model.SharePrx = SharePrx
    del SharePrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright 2009 Glencoe Software, Inc. All rights reserved.
#    Use is subject to license terms supplied in LICENSE.txt
#

"""
Concrete implementations of the omero.grid.Column
type hierarchy which know how to convert themselves
to PyTables types.
"""

from builtins import zip
from builtins import range
from builtins import object
from future.utils import native, bytes_to_native_str, isbytes
import omero
import Ice
import IceImport
IceImport.load("omero_Tables_ice")
python_sys = __import__("sys")  # Python sys

try:
    import numpy
    tables = __import__("tables")  # Pytables
    has_pytables = True
    if hasattr(tables, "open_file"):
        has_pytables3 = True
    else:
        has_pytables3 = False
except ImportError:
    has_pytables = False


def columns2definition(cols):
    """
    Takes a list of columns and converts them into a map
    from names to tables.* column descriptors
    """
    definition = {}
    for i in range(len(cols)):
        column = cols[i]
        instance = column.descriptor(pos=i)
        if column.name in definition:
            raise omero.ApiUsageException(
                None, None, "Duplicate column name: %s" % column.name)
        definition[column.name] = instance
        # Descriptions are handled separately
    return definition


class AbstractColumn(object):
    """
    Base logic for all columns
    """

    def __init__(self):
        # Note: don't rely on any properties such as self.name being set if
        # this has been called through Ice
        d = self.descriptor(None)
        if isinstance(d, tables.IsDescription):
            cols = d.columns
            try:
                del cols["_v_pos"]
            except KeyError:
                pass
            self._types = [None] * len(cols)
            self._subnames = [None] * len(cols)
            for k, v in list(cols.items()):
                self._types[v._v_pos] = v.recarrtype
                self._subnames[v._v_pos] = "/" + k

        else:
            self._types = [d.recarrtype]
            self._subnames = [""]

    def settable(self, tbl):
        """
        Called by tables.py when first initializing columns.
        Can be used to complete further initialization.
        """
        self.__table = tbl

    def append(self, tbl):
        """
        Called by tables.py to give columns. By default, does nothing.
        """
        pass

    def readCoordinates(self, tbl, rowNumbers):
        if rowNumbers is None or len(rowNumbers) == 0:
            rows = tbl.read()
        else:
            if has_pytables3:
                rows = tbl.read_coordinates(rowNumbers, field=self.name)
            else:
                rows = tbl.readCoordinates(rowNumbers, field=self.name)
        self.fromrows(rows, field_only=True)

    def read(self, tbl, start, stop):
        rows = tbl.read(start, stop, field=self.name)
        self.fromrows(rows, field_only=True)

    def getsize(self):
        """
        Any method which does not use the "values" field
        will need to override this method.
        """
        if self.values is None:
            return None
        else:
            return len(self.values)

    def setsize(self, size):
        """
        Any method which does not use the "values" field
        will need to override this method.
        """
        if size is None:
            self.values = None
        else:
            self.values = [None for x in range(size)]

    def arrays(self):
        """
        Any method which does not use the "values" field
        will need to override this method.
        """
        return [self.values]

    def dtypes(self):
        """
        Override this method if descriptor() doesn't return the correct data
        type/size at initialisation- this is mostly a problem for array types
        """
        names = [self.name + sn for sn in self._subnames]
        return list(zip(names, self._types))

    def fromrows(self, rows, field_only=False):
        """
        Any method which does not use the "values" field
        will need to override this method.
        """

        if not field_only:
            rows = rows[self.name]

        self.values = rows

        # WORKAROUND:
        # http://www.zeroc.com/forums/bug-reports/4165-icepy-can-not-handle-buffers-longs-i64.html#post20468
        # see ticket:1951 and #2160
        # d = self.recarrtypes[0][1]
        # Disabled until Ice 3.4
        # if isinstance(d, str):
        #     d = numpy.dtype(d)
        # if d.kind == "S" or (d.kind == "i" and d.itemsize == "8"):
        self.values = self.values.tolist()


class FileColumnI(AbstractColumn, omero.grid.FileColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.FileColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)


class ImageColumnI(AbstractColumn, omero.grid.ImageColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.ImageColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)


class WellColumnI(AbstractColumn, omero.grid.WellColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.WellColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)


class PlateColumnI(AbstractColumn, omero.grid.PlateColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.PlateColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)

class DatasetColumnI(AbstractColumn, omero.grid.DatasetColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.DatasetColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)


class RoiColumnI(AbstractColumn, omero.grid.RoiColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.RoiColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)


class BoolColumnI(AbstractColumn, omero.grid.BoolColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.BoolColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.BoolCol(pos=pos)


class DoubleColumnI(AbstractColumn, omero.grid.DoubleColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.DoubleColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Float64Col(pos=pos)


class LongColumnI(AbstractColumn, omero.grid.LongColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.LongColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def descriptor(self, pos):
        return tables.Int64Col(pos=pos)


class StringColumnI(AbstractColumn, omero.grid.StringColumn):
    """
    StringColumns are actually numpy dtype 'S': "zero-terminated bytes (not recommended)"
    https://github.com/ome/omero-py/blob/v5.6.dev8/src/omero/columns.py#L269
    https://docs.scipy.org/doc/numpy-1.15.1/reference/arrays.dtypes.html#specifying-and-constructing-data-types
    In any case HDF5 doesn't seem to properly support unicode,
    and numexpr doesn't even pretend to support it:
    - https://github.com/PyTables/PyTables/issues/499
    - https://github.com/pydata/numexpr/issues/142
    - https://github.com/pydata/numexpr/issues/150
    - https://github.com/pydata/numexpr/issues/263
    - https://github.com/pydata/numexpr/blob/v2.7.0/numexpr/necompiler.py#L340-L341

    > import numexpr
    > a = "£"
    > numexpr.evaluate('a=="£"')
    ValueError: unknown type str32
    > b = "£".encode()
    > numexpr.evaluate('b=="£"')
    UnicodeEncodeError: 'ascii' codec can't encode character '\xa3' in position 0: ordinal not in range(128)

    You should be able to store/load unicode data but you can't use
    unicode in a where condition
    """

    def __init__(self, name="Unknown", *args):
        omero.grid.StringColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def settable(self, tbl):
        AbstractColumn.settable(self, tbl)
        self.size = getattr(tbl.cols, self.name).dtype.itemsize

    def arrays(self):
        """
        Check for strings longer than the initialised column width
        This will always return bytes
        """
        if python_sys.version_info >= (3, 0, 0):
            bytevalues = [v.encode() for v in self.values]
        else:
            bytevalues = self.values
        for bv in bytevalues:
            if len(bv) > self.size:
                raise omero.ValidationException(
                    None, None,
                    "Maximum string (byte) length in column %s is %d" %
                    (self.name, self.size))
        return [bytevalues]

    def dtypes(self):
        """
        Overriding to correct for size.
        (Testing suggests this may not be necessary, the size appears to be
        correctly set at initialisation)
        """
        return [(self.name, "S", self.size)]

    def descriptor(self, pos):
        # During initialization, size might be zero
        # to prevent exceptions we temporarily assume size 1
        if pos is None:
            return tables.StringCol(pos=pos, itemsize=1)
        if self.size < 1:
            raise omero.ApiUsageException(
                None, None, "String size must be > 0 (Column: %s)"
                % self.name)
        return tables.StringCol(pos=pos, itemsize=self.size)

    def fromrows(self, rows, field_only=False):
        AbstractColumn.fromrows(self, rows, field_only=field_only)
        for i, val in enumerate(self.values):
            if isbytes(val):
                self.values[i] = bytes_to_native_str(val)


class AbstractArrayColumn(AbstractColumn):
    """
    Additional base logic for array columns
    """

    def __init__(self):
        AbstractColumn.__init__(self)

    def settable(self, tbl):
        AbstractColumn.settable(self, tbl)

        # Pytables 2.1 has the array size in Column.dtype.shape
        # shape = getattr(tbl.cols, self.name).dtype.shape
        # self.size = shape[0]

        # Pytables 2.2 and later replaced this with Column.shape
        # shape = getattr(tbl.cols, self.name).shape
        # assert(len(shape) == 2)
        # self.size = shape[1]

        # http://www.pytables.org/trac-bck/ticket/231
        # http://www.pytables.org/trac-bck/ticket/232
        # TODO: Clean this up

        # Taken from http://www.pytables.org/trac-bck/changeset/4176
        column = getattr(tbl.cols, self.name)
        self.size = column.descr._v_dtypes[column.name].shape[0]

    def arrays(self):
        """
        Arrays of size 1 have to be converted to scalars, otherwise the
        column-to-row conversion in HdfStorage.append() will fail.
        This is messy, but I can't think of a better way.
        """
        for v in self.values:
            if len(v) != self.size:
                raise omero.ValidationException(
                    None, None, "Column %s requires arrays of length %d" %
                    (self.name, self.size))

        if self.size == 1:
            return [[v[0] for v in self.values]]
        return [self.values]

    def dtypes(self):
        """
        Overriding to correct for size.
        """
        return [(self.name, self._types[0], self.size)]


class FloatArrayColumnI(AbstractArrayColumn, omero.grid.FloatArrayColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.FloatArrayColumn.__init__(self, name, *args)
        AbstractArrayColumn.__init__(self)

    def descriptor(self, pos):
        # During initialization, size might be zero
        if pos is None:
            return tables.Float32Col(pos=pos)
        if self.size < 1:
            raise omero.ApiUsageException(
                None, None, "Array length must be > 0 (Column: %s)"
                % self.name)
        return tables.Float32Col(pos=pos, shape=self.size)


class DoubleArrayColumnI(AbstractArrayColumn, omero.grid.DoubleArrayColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.DoubleArrayColumn.__init__(self, name, *args)
        AbstractArrayColumn.__init__(self)

    def descriptor(self, pos):
        # During initialization, size might be zero
        if pos is None:
            return tables.Float64Col(pos=pos)
        if self.size < 1:
            raise omero.ApiUsageException(
                None, None, "Array length must be > 0 (Column: %s)"
                % self.name)
        return tables.Float64Col(pos=pos, shape=self.size)


class LongArrayColumnI(AbstractArrayColumn, omero.grid.LongArrayColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.LongArrayColumn.__init__(self, name, *args)
        AbstractArrayColumn.__init__(self)

    def descriptor(self, pos):
        # During initialization, size might be zero
        if pos is None:
            return tables.Int64Col(pos=pos)
        if self.size < 1:
            raise omero.ApiUsageException(
                None, None, "Array length must be > 0 (Column: %s)"
                % self.name)
        return tables.Int64Col(pos=pos, shape=self.size)


class MaskColumnI(AbstractColumn, omero.grid.MaskColumn):

    def __init__(self, name="Unknown", *args):
        omero.grid.MaskColumn.__init__(self, name, *args)
        AbstractColumn.__init__(self)

    def __noneorsame(self, a, b):
        if a is None:
            if b is None:
                return
        # a not none
        if b is not None:
            if len(a) == len(b):
                return
        raise omero.ValidationException(None, None, "Columns don't match")

    def __sanitycheck(self):
        self.__noneorsame(self.imageId, self.theZ)
        self.__noneorsame(self.imageId, self.theT)
        self.__noneorsame(self.imageId, self.x)
        self.__noneorsame(self.imageId, self.y)
        self.__noneorsame(self.imageId, self.w)
        self.__noneorsame(self.imageId, self.h)
        self.__noneorsame(self.imageId, self.bytes)

    def descriptor(self, pos):
        class MaskDescription(tables.IsDescription):
            _v_pos = pos
            i = tables.Int64Col(pos=0)
            z = tables.Int32Col(pos=1)
            t = tables.Int32Col(pos=2)
            x = tables.Float64Col(pos=3)
            y = tables.Float64Col(pos=4)
            w = tables.Float64Col(pos=5)
            h = tables.Float64Col(pos=6)
        return MaskDescription()

    def arrays(self):
        self.__sanitycheck()
        a = [
            self.imageId,
            self.theZ,
            self.theT,
            self.x,
            self.y,
            self.w,
            self.h,
            ]
        return a

    def getsize(self):
        self.__sanitycheck()
        if self.imageId is None:
            return None
        else:
            return len(self.imageId)

    def setsize(self, size):
        if size is None:
            self.imageId = None
            self.theZ = None
            self.theT = None
            self.x = None
            self.y = None
            self.w = None
            self.h = None
        else:
            dts = self.dtypes()
            self.imageId = numpy.zeroes(size, dtype=dts[0])
            self.theZ = numpy.zeroes(size, dtype=dts[1])
            self.theT = numpy.zeroes(size, dtype=dts[2])
            self.x = numpy.zeroes(size, dtype=dts[3])
            self.y = numpy.zeroes(size, dtype=dts[4])
            self.w = numpy.zeroes(size, dtype=dts[5])
            self.h = numpy.zeroes(size, dtype=dts[6])

    def readCoordinates(self, tbl, rowNumbers):
        self.__sanitycheck()
        # calls fromrows
        AbstractColumn.readCoordinates(self, tbl, rowNumbers)
        masks = self._getmasks(tbl)
        if rowNumbers is None or len(rowNumbers) == 0:
            rowNumbers = list(range(masks.nrows))
        self.getbytes(masks, rowNumbers)

    def read(self, tbl, start, stop):
        self.__sanitycheck()
        # calls fromrows
        AbstractColumn.read(self, tbl, start, stop)
        masks = self._getmasks(tbl)
        rowNumbers = list(range(start, stop))
        self.getbytes(masks, rowNumbers)

    def getbytes(self, masks, rowNumbers):
        self.bytes = []
        for idx in rowNumbers:
            self.bytes.append(masks[idx].tolist())

    def fromrows(self, rows, field_only=False):

        if not field_only:
            rows = rows[self.name]

        # WORKAROUND:
        # http://www.zeroc.com/forums/bug-reports/4165-icepy-can-not-handle-buffers-longs-i64.html#post20468
        self.imageId = rows["i"].tolist()
        self.theZ = rows["z"].tolist()  # ticket:1665
        self.theT = rows["t"].tolist()  # ticket:1665
        self.x = rows["x"]
        self.y = rows["y"]
        self.w = rows["w"]
        self.h = rows["h"]

    def append(self, tbl):
        self.__sanitycheck()
        masks = self._getmasks(tbl)
        for x in self.bytes:
            if isinstance(x, list):
                # This occurs primarily in testing.
                masks.append(numpy.array(x, dtype=tables.UInt8Atom()))
            else:
                masks.append(numpy.fromstring(x, count=len(x),
                                              dtype=tables.UInt8Atom()))

    def _getmasks(self, tbl):
        n = tbl._v_name
        f = tbl._v_file
        p = tbl._v_parent
        # http://doc.zeroc.com/display/Ice/Basic+Types
        # Ice::Byte can be -128 to 127 OR 0 to 255, but using UInt8 for the
        # moment
        try:
            masks = getattr(p, "%s_masks" % n)
        except tables.NoSuchNodeError:
            if has_pytables3:
                masks = f.create_vlarray(p, "%s_masks" % n, tables.UInt8Atom())
            else:
                masks = f.createVLArray(p, "%s_masks" % n, tables.UInt8Atom())
        return masks

# Helpers
# ========================================================================


# Conversion classes are for omero.model <--> ome.model only (no python)
class ObjectFactory(Ice.ObjectFactory):

    def __init__(self, cls, f):
        try:
            self.id = cls.ice_staticId()
        except Exception:
            pass

        self.f = f

    def create(self, string):
        return self.f()

    def destroy(self):
        pass

    def register(self, ic):
        ic.addObjectFactory(self, self.id)


# Object factories
# =========================================================================

ObjectFactories = {
    FileColumnI: ObjectFactory(FileColumnI, lambda: FileColumnI()),
    ImageColumnI: ObjectFactory(ImageColumnI, lambda: ImageColumnI()),
    RoiColumnI: ObjectFactory(RoiColumnI, lambda: RoiColumnI()),
    WellColumnI: ObjectFactory(WellColumnI, lambda: WellColumnI()),
    PlateColumnI: ObjectFactory(PlateColumnI, lambda: PlateColumnI()),
    DatasetColumnI: ObjectFactory(DatasetColumnI, lambda: DatasetColumnI()),
    BoolColumnI: ObjectFactory(BoolColumnI, lambda: BoolColumnI()),
    DoubleColumnI: ObjectFactory(DoubleColumnI, lambda: DoubleColumnI()),
    LongColumnI: ObjectFactory(LongColumnI, lambda: LongColumnI()),
    StringColumnI: ObjectFactory(StringColumnI, lambda: StringColumnI()),
    FloatArrayColumnI: ObjectFactory(
        FloatArrayColumnI, lambda: FloatArrayColumnI()),
    DoubleArrayColumnI: ObjectFactory(
        DoubleArrayColumnI, lambda: DoubleArrayColumnI()),
    LongArrayColumnI: ObjectFactory(
        LongArrayColumnI, lambda: LongArrayColumnI()),
    MaskColumnI: ObjectFactory(MaskColumnI, lambda: MaskColumnI())
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Utility methods for manipulating roi.
"""
from __future__ import division

from builtins import map
from past.utils import old_div
from numpy import asarray, int32, math, zeros, hstack, vstack
import omero.util.script_utils as script_utils


def get_line_data(pixels, x1, y1, x2, y2, line_w=2, the_z=0, the_c=0, the_t=0):
    """
    Grabs pixel data covering the specified line, and rotates it horizontally
    so that x1,y1 is to the left,
    Returning a numpy 2d array. Used by Kymograph.py script.
    Uses PIL to handle rotating and interpolating the data. Converts to numpy
    to PIL and back (may change dtype.)

    @param pixels:          PixelsWrapper object
    @param x1, y1, x2, y2:  Coordinates of line
    @param line_w:          Width of the line we want
    @param the_z:           Z index within pixels
    @param the_c:           Channel index
    @param the_t:           Time index
    """

    size_x = pixels.getSizeX()
    size_y = pixels.getSizeY()

    line_x = x2-x1
    line_y = y2-y1

    rads = math.atan2(line_x, line_y)

    # How much extra Height do we need, top and bottom?
    extra_h = abs(math.sin(rads) * line_w)
    bottom = int(max(y1, y2) + old_div(extra_h,2))
    top = int(min(y1, y2) - old_div(extra_h,2))

    # How much extra width do we need, left and right?
    extra_w = abs(math.cos(rads) * line_w)
    left = int(min(x1, x2) - extra_w)
    right = int(max(x1, x2) + extra_w)

    # What's the larger area we need? - Are we outside the image?
    pad_left, pad_right, pad_top, pad_bottom = 0, 0, 0, 0
    if left < 0:
        pad_left = abs(left)
        left = 0
    x = left
    if top < 0:
        pad_top = abs(top)
        top = 0
    y = top
    if right > size_x:
        pad_right = right-size_x
        right = size_x
    w = int(right - left)
    if bottom > size_y:
        pad_bottom = bottom-size_y
        bottom = size_y
    h = int(bottom - top)
    tile = (x, y, w, h)

    # get the Tile
    plane = pixels.getTile(the_z, the_c, the_t, tile)

    # pad if we wanted a bigger region
    if pad_left > 0:
        data_h, data_w = plane.shape
        pad_data = zeros((data_h, pad_left), dtype=plane.dtype)
        plane = hstack((pad_data, plane))
    if pad_right > 0:
        data_h, data_w = plane.shape
        pad_data = zeros((data_h, pad_right), dtype=plane.dtype)
        plane = hstack((plane, pad_data))
    if pad_top > 0:
        data_h, data_w = plane.shape
        pad_data = zeros((pad_top, data_w), dtype=plane.dtype)
        plane = vstack((pad_data, plane))
    if pad_bottom > 0:
        data_h, data_w = plane.shape
        pad_data = zeros((pad_bottom, data_w), dtype=plane.dtype)
        plane = vstack((plane, pad_data))

    pil = script_utils.numpy_to_image(plane, (plane.min(), plane.max()), int32)

    # Now need to rotate so that x1,y1 is horizontally to the left of x2,y2
    to_rotate = 90 - math.degrees(rads)

    if x1 > x2:
        to_rotate += 180
    # filter=Image.BICUBIC see
    # http://www.ncbi.nlm.nih.gov/pmc/articles/PMC2172449/
    rotated = pil.rotate(to_rotate, expand=True)
    # rotated.show()

    # finally we need to crop to the length of the line
    length = int(math.sqrt(math.pow(line_x, 2) + math.pow(line_y, 2)))
    rot_w, rot_h = rotated.size
    crop_x = old_div((rot_w - length),2)
    crop_x2 = crop_x + length
    crop_y = old_div((rot_h - line_w),2)
    crop_y2 = crop_y + line_w
    cropped = rotated.crop((crop_x, crop_y, crop_x2, crop_y2))
    return asarray(cropped)


def points_string_to_xy_list(string):
    """
    Method for converting the string returned from
    omero.model.ShapeI.getPoints() into list of (x,y) points (floats)
    e.g. "points[309,427, 366,503, 190,491]"
    """
    point_lists = string.strip().split("points")
    if len(point_lists) < 2:
        if len(point_lists) == 1 and point_lists[0]:
            xys = point_lists[0].split()
            xy_list = [tuple(map(float, xy.split(','))) for xy in xys]
            return xy_list
        raise ValueError("Unrecognised ROI shape 'points' string: %s" % string)

    first_list = point_lists[1]
    xy_list = []
    for xy in first_list.strip(" []").split(", "):
        x, y = xy.split(",")
        xy_list.append((float(x.strip()), float(y.strip())))
    return xy_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is an import-only file providing a mechanism for other files to
# import a range of modules in a controlled way. It could be made to pass
# flake8 but given its simplicity it is being marked as noqa for now.
#
# flake8: noqa

import omero.clients
